--
-- Specialisation for MB Trac 1600 Turbo
-- 
-- Author: Mofa-Killer, Face

Vehicle.registerJointType("attachableFrontloader");

MBTrac1600Turbo = {};

function MBTrac1600Turbo.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function MBTrac1600Turbo:load(xmlFile)

    self.backHydraulic = {};
	self.backHydraulic.rootArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#rootArm"));
	self.backHydraulic.bottomArm = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.backHydraulic#bottomArm"));
	self.frontAttacherJoint = {};
	self.frontAttacherJoint.node = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.trailerAttacherJoints#front"));
	
	local wiperAnimRootNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.wiper#rootNode"));
    self.wiperAnimCharSet = 0;
    if wiperAnimRootNode ~= nil and wiperAnimRootNode ~= 0 then
        self.wiperAnimCharSet = getAnimCharacterSet(wiperAnimRootNode);
        if self.wiperAnimCharSet ~= 0 then
            local clip = getAnimClipIndex(self.wiperAnimCharSet, getXMLString(xmlFile, "vehicle.wiper#clip"));
            assignAnimTrackClip(self.wiperAnimCharSet, 0, clip);
            setAnimTrackLoopState(self.wiperAnimCharSet, 0, true);
            local wiperAnimSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.wiper#speedScale"), 1);
			setAnimTrackSpeedScale(self.wiperAnimCharSet, 0, wiperAnimSpeedScale);
            self.wiperAnimDuration = getAnimClipDuration(self.wiperAnimCharSet, clip);
        end;
    end;
	self.isWiperActive = false;
	self.finishWiper = true;
    
	local gaspedalNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.gaspedal#index"));
    if gaspedalNode ~= nil then
        self.gaspedal = {};
        self.gaspedal.node = gaspedalNode;    
		self.gaspedal.maxSpeed = getXMLInt(xmlFile, "vehicle.gaspedal#maxSpeed");
		local x1,y1,z1 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#minRot"));
		local x2,y2,z2 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#maxRot"));		
		self.gaspedal.rotMin = {math.rad(Utils.getNoNil(x1,0)),math.rad(Utils.getNoNil(y1,0)),math.rad(Utils.getNoNil(z1,0))};
		self.gaspedal.rotMax = {math.rad(Utils.getNoNil(x2,0)),math.rad(Utils.getNoNil(y2,0)),math.rad(Utils.getNoNil(z2,0))};
    end;
	
    local bremspedalNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.bremspedal#index"));
    if bremspedalNode ~= nil then
        self.bremspedal = {};
        self.bremspedal.node = bremspedalNode;
		local x1,y1,z1 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#minRot"));
		local x2,y2,z2 = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.gaspedal#maxRot"));
		self.bremspedal.rotMin = {math.rad(Utils.getNoNil(x1,0)),math.rad(Utils.getNoNil(y1,0)),math.rad(Utils.getNoNil(z1,0))};			
		self.bremspedal.rotMax = {math.rad(Utils.getNoNil(x2,0)),math.rad(Utils.getNoNil(y2,0)),math.rad(Utils.getNoNil(z2,0))};
        self.bremspedal.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.bremspedal#rotTime"), 1)*1000;
        self.bremspedalIsActive = false;
    end;	
	
    local blinkerhebelNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.blinkerhebel#index"));
    if blinkerhebelNode ~= nil then
        self.blinkerhebel = {};
        self.blinkerhebel.node = blinkerhebelNode;
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#minRot"));
        self.blinkerhebel.minRot = {};
        self.blinkerhebel.minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#rotLeft"));
        self.blinkerhebel.leftRot = {};
        self.blinkerhebel.leftRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.leftRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.leftRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.blinkerhebel#rotRight"));
        self.blinkerhebel.rightRot = {};
        self.blinkerhebel.rightRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.blinkerhebel.rightRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.blinkerhebel.rightRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

        self.blinkerhebel.rotTime = Utils.getNoNil(getXMLString(xmlFile, "vehicle.blinkerhebel#rotTime"), 1)/1000;
		self.blinkerhebel.lastDir = 1;
    end;
	
	self.drehzahlnadel = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drehzahlnadel#index"));
	self.tanknadel = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tanknadel#index"));
	self.drehzahlnadelLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.drehzahlnadel#light"));
	self.tanknadelLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tanknadel#light"));
	self.tankicon = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.tankicon#index"));
	self.batterieicon = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.batterieicon#index"));
	self.blinkericon = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.blinkericon#index"));
	self.konsole = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.konsole#index"));
	self.sitz = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.sitz#index"));
	
	self.controlPanel1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.controlPanels.controlPanel1#index" ));
	self.controlPanel2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.controlPanels.controlPanel2#index" ));
	
	self.updateJoint = false;
end;

function MBTrac1600Turbo:delete()
end;

function MBTrac1600Turbo:readStream(streamId, connection)
end;

function MBTrac1600Turbo:writeStream(streamId, connection)
end;

function MBTrac1600Turbo:mouseEvent(posX, posY, isDown, isUp, button)
end;

function MBTrac1600Turbo:keyEvent(unicode, sym, modifier, isDown)
end;

function MBTrac1600Turbo:update(dt)
    if self:getIsActive() then
		Cylindered.updateMovingPart(self, self.movingParts[1]);
        
        if self.backHydraulic ~= nil then
			local x, y, z = getRotation(self.backHydraulic.bottomArm);
			setRotation(self.backHydraulic.rootArm, x, y, z);
		end;
        
        for _, part in pairs(self.movingParts) do
			part.isDirty = true;
		end;
        
        local sx,sy,sz = getTranslation(self.sitz);
        local _,ry,_ = getTranslation(self.wheels[1].repr);
        setTranslation(self.sitz, sx, (ry+2.0895) /3, sz);   		
	end;
	    
    if self.invertedDrivingDirection then
        self.B3.md = -1;
        setTranslation(self.controlPanel1, -0.45, 2.84, -0.12);
        setRotation(self.controlPanel1, 70, -70, -70);
        setTranslation(self.controlPanel2, -0.22, 2.84, -0.117);
        setRotation(self.controlPanel2, 70, -70, -70);
    else
        self.B3.md = 1;
        setTranslation(self.controlPanel1, -0.635, 2.25, 0.935);
        setRotation(self.controlPanel1, 0, -200, 0);
        setTranslation(self.controlPanel2, -0.627, 2.55, 0.915);
        setRotation(self.controlPanel2, 0, -200, 0);
    end;	
end;

function MBTrac1600Turbo:updateTick(dt)

    if self:getIsActive() then	
		if not self.finishWiper then
			if getAnimTrackTime(self.wiperAnimCharSet, 0) % self.wiperAnimDuration <= 100 then
				setAnimTrackTime(self.wiperAnimCharSet, 0, 0.0);
				disableAnimTrack(self.wiperAnimCharSet, 0);
				self.finishWiper = true;
			end;
		end;
		
		if g_currentMission.environment.lastRainScale > 0.1 and g_currentMission.environment.timeSinceLastRain < 30 then
			if not self.isWiperActive then
				enableAnimTrack(self.wiperAnimCharSet, 0);
				self.isWiperActive = true;
			end;
		else
			if self.isWiperActive then
				self.isWiperActive = false;
				self.finishWiper = false;
			end;
		end;
		
		-- check if currentcam is indoorcam
		if self.camIndex == 2 then
		
			if self.gaspedal ~= nil then		
				local percent = (self.lastSpeed*self.speedDisplayScale*3600) / self.gaspedal.maxSpeed;
				local minRot = self.gaspedal.rotMin;
				local maxRot = self.gaspedal.rotMax;
					
				local x = minRot[1] + (maxRot[1] - minRot[1]) * percent;
				local y = minRot[2] + (maxRot[2] - minRot[2]) * percent;
				local z = minRot[3] + (maxRot[3] - minRot[3]) * percent;
				setRotation(self.gaspedal.node, x, y, z);
			end;
			
			-- check for braking
			if self.bremspedal ~= nil then
				local x, y, z = getRotation(self.bremspedal.node);
				local rot = {x,y,z};
				local direction = 1;
				if self.invertedDrivingDirection then
				    direction = -1;
				else
				    direction = 1;
				end;
				local braking = math.abs(Utils.sign(self.lastAcceleration)+ (-self.movingDirection * direction)) == 2;
				local newRot = Utils.getMovedLimitedValues(rot, self.bremspedal.rotMax, self.bremspedal.rotMin, 3, self.bremspedal.rotTime, dt, braking);
				setRotation(self.bremspedal.node, unpack(newRot));
			end;
			
			if self.blinkerhebel ~= nil then
				local dir = 1;
				local hasRotChanged = false;
				local destRot = self.blinkerhebel.minRot;
				if not (self.B3.dirRight[1].a and self.B3.dirLeft[1].a) then
					if self.B3.dirLeft[1].a then
						destRot = self.blinkerhebel.leftRot;
						self.blinkerhebel.lastDir = 1;		
					elseif self.B3.dirRight[1].a then
						destRot = self.blinkerhebel.rightRot;
						self.blinkerhebel.lastDir = -1;
						dir = -1;
					else
						if not self.B3.dirRight[1].a and not self.B3.dirLeft[1].a then
							if self.blinkerhebel.lastDir ~= -1 then
								dir = -1;
								if self.blinkerhebel.lastDir ~= 1 then
								    dir = 1;
								end;
							end;
						end;
					end;
				else
					local x,y,z = getRotation(self.blinkerhebel.node);
					if y > 0 then
						dir = -1;
					end;
				end;
				
				local rot = {getRotation(self.blinkerhebel.node)};
				for i=1, 3 do
					
					local newRot = AnimatedVehicle.getMovedLimitedValue(rot[i], destRot[i], self.blinkerhebel.rotTime*dir, dt);
					if rot[i] ~= newRot then
						hasRotChanged = true;
						rot[i] = newRot;
					end;
				end;
				if hasRotChanged then
					setRotation(self.blinkerhebel.node, unpack(rot));
				end;
			end;			
		else
			setRotation(self.gaspedal.node, unpack(self.gaspedal.rotMin));
			setRotation(self.bremspedal.node, unpack(self.bremspedal.rotMin));
		end;
		
		if self.isMotorStarted then
			local kmh = math.min(100, math.max(0, self.lastSpeed*self.speedDisplayScale*3600));
			local rotateDrehzahlnadel = (((kmh / 25)*100) +10);
			local currentFuelPercentage = 0;
			local fuelWarnPercentage = 20;
			if self.fuelCapacity > 0 then
				currentFuelPercentage = (self.fuelFillLevel / self.fuelCapacity + 0.0001) * 100;
			end;
			if currentFuelPercentage < fuelWarnPercentage then
				setVisibility(self.tankicon, true);
			else
				setVisibility(self.tankicon, false);
			end;
			setVisibility(self.batterieicon, false);
			local rotateTanknadel = ((currentFuelPercentage / 111) * 100);
			if self.currentLight > 0 then
				setVisibility(self.drehzahlnadel, false);
				setVisibility(self.tanknadel, false);
				setVisibility(self.drehzahlnadelLight, true);
				setVisibility(self.tanknadelLight, true);    
				setRotation(self.drehzahlnadelLight, 0, Utils.degToRad( - rotateDrehzahlnadel), 0);
				setRotation(self.tanknadelLight, 0, Utils.degToRad(rotateTanknadel), 0);
			else
				setVisibility(self.drehzahlnadel, true);
				setVisibility(self.tanknadel, true);
				setVisibility(self.drehzahlnadelLight, false);
				setVisibility(self.tanknadelLight, false);
				setRotation(self.drehzahlnadel, 0, Utils.degToRad( - rotateDrehzahlnadel), 0);
				setRotation(self.tanknadel, 0, Utils.degToRad(rotateTanknadel), 0);
			end;
		else
			setRotation(self.drehzahlnadel, 0, 0, 0);
			setRotation(self.tanknadel, 0, 0, 0);
			setRotation(self.drehzahlnadelLight, 0, 0, 0);
			setRotation(self.tanknadelLight, 0, 0, 0);
			setVisibility(self.batterieicon, true);
			setVisibility(self.tankicon, false);
		end;
	end;
	
	if self.B3.wl or self.B3.dirLeft[1].a or self.B3.dirRight[1].a  then
		setVisibility(self.blinkericon,not self.B3.bls);
	else 
		setVisibility(self.blinkericon, false);
	end;
end;

function MBTrac1600Turbo:draw()
end;

function MBTrac1600Turbo:attachImplement(implement)

	local jointType = implement.object.attacherJoint.jointType;
	local jointIndex = implement.jointDescIndex;

	if jointType == Vehicle.JOINTTYPE_IMPLEMENT then
		if jointIndex == 2 then
			setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, true);
		end;
	end;
	
	if jointIndex == 3 then
	   setVisibility(self.konsole, true);
	end;
	
	self.updateJoint = true;
end;

function MBTrac1600Turbo:detachImplement(implementIndex)

	local implement = self.attachedImplements[implementIndex];
	local jointIndex = implement.jointDescIndex;

	if jointIndex == 2 then
		setVisibility(self.attacherJoints[jointIndex].topArm.rotationNode, false);
	end;
	
	if jointIndex == 3 then
		setVisibility(self.konsole, false);
	end;
end;

function MBTrac1600Turbo:validateAttacherJoint(implement, jointDesc, dt)
    if self.updateJoint then
        self.updateJoint = false;
        return true;
    end;
    return false;
end;

function MBTrac1600Turbo:onLeave()
	if self.isWiperActive then
		disableAnimTrack(self.wiperAnimCharSet, 0);
	end;
	
	if self.isMotorStarted then
	   setVisibility(self.batterieicon, false);
	else
	   setVisibility(self.batterieicon, true);
	end;
end;

function MBTrac1600Turbo:onEnter()
	if self.isWiperActive then
		enableAnimTrack(self.wiperAnimCharSet, 0);
	end;
	
	if self.isMotorStarted then
	   setVisibility(self.batterieicon, false);
	else
	   setVisibility(self.batterieicon, true);
	end;
end;

